@@ -9,57 +9,25 @@ in settings.py for arguments that you want always passed to nose.
 
 """
 from __future__ import print_function, unicode_literals
+
 import os
 import sys
-from optparse import make_option, NO_DEFAULT
+from importlib import import_module
+from optparse import NO_DEFAULT
 from types import MethodType
 
-import django
+from django import setup
+from django.apps import apps
 from django.conf import settings
 from django.core import exceptions
-from django.core.management.base import BaseCommand
 from django.core.management.color import no_style
 from django.core.management.commands.loaddata import Command
 from django.db import connections, transaction, DEFAULT_DB_ALIAS
-
-try:
-    from django.db.backends.base.creation import BaseDatabaseCreation
-except ImportError:
-    # Django < 1.7
-    from django.db.backends.creation import BaseDatabaseCreation
-
-try:
-    from importlib import import_module
-except ImportError:
-    # Django < 1.7 and Python < 2.7
-    from django.utils.importlib import import_module
-
-try:
-    from django.apps import apps
-except ImportError:
-    # Django < 1.7
-    from django.db.models.loading import cache as apps
-
-import nose.core
+from django.test.runner import DiscoverRunner
 
 from django_nose.plugin import DjangoSetUpPlugin, ResultPlugin, TestReorderer
 from django_nose.utils import uses_mysql
-
-try:
-    any
-except NameError:
-    def any(iterable):
-        for element in iterable:
-            if element:
-                return True
-        return False
-
-try:
-    from django.test.runner import DiscoverRunner
-except ImportError:
-    # Django < 1.8
-    from django.test.simple import DjangoTestSuiteRunner as DiscoverRunner
-
+import nose.core
 
 __all__ = ('BasicNoseRunner', 'NoseTestSuiteRunner')
 
@@ -77,18 +45,6 @@ def translate_option(opt):
     return OPTION_TRANSLATION.get(opt, opt)
 
 
-# Django v1.2 does not have a _get_test_db_name() function.
-if not hasattr(BaseDatabaseCreation, '_get_test_db_name'):
-    def _get_test_db_name(self):
-        TEST_DATABASE_PREFIX = 'test_'
-
-        if self.connection.settings_dict['TEST_NAME']:
-            return self.connection.settings_dict['TEST_NAME']
-        return TEST_DATABASE_PREFIX + self.connection.settings_dict['NAME']
-
-    BaseDatabaseCreation._get_test_db_name = _get_test_db_name
-
-
 def _get_plugins_from_settings():
     plugins = (list(getattr(settings, 'NOSE_PLUGINS', [])) +
                ['django_nose.plugin.TestReorderer'])
@@ -116,193 +72,151 @@ def _get_plugins_from_settings():
         yield p_class()
 
 
-def _get_options():
-    """Return all nose options that don't conflict with django options."""
-    cfg_files = nose.core.all_config_files()
-    manager = nose.core.DefaultPluginManager()
-    config = nose.core.Config(env=os.environ, files=cfg_files, plugins=manager)
-    config.plugins.addPlugins(list(_get_plugins_from_settings()))
-    options = config.getParser()._get_all_options()
+class BaseRunner(DiscoverRunner):
+    """Runner that translates nose optparse arguments to argparse.
 
-    # copy nose's --verbosity option and rename to --nose-verbosity
-    verbosity = [o for o in options if o.get_opt_string() == '--verbosity'][0]
-    verbosity_attrs = dict((attr, getattr(verbosity, attr))
-                           for attr in verbosity.ATTRS
-                           if attr not in ('dest', 'metavar'))
-    options.append(make_option('--nose-verbosity',
-                               dest='nose_verbosity',
-                               metavar='NOSE_VERBOSITY',
-                               **verbosity_attrs))
+    Django 1.8 and later uses argparse.ArgumentParser. Nose's optparse
+    arguments need to be translated to this format, so that the Django
+    command line parsing will pass. This parsing is (mostly) thrown out,
+    and reassembled into command line arguments for nose to reparse.
+    """
 
-    # Django 1.6 introduces a "--pattern" option, which is shortened into "-p"
-    # do not allow "-p" to collide with nose's "--plugins" option.
-    plugins_option = [
-        o for o in options if o.get_opt_string() == '--plugins'][0]
-    plugins_option._short_opts.remove('-p')
+    # Don't pass the following options to nosetests
+    django_opts = [
+        '--noinput', '--liveserver', '-p', '--pattern', '--testrunner',
+        '--settings',
+        # 1.8 arguments
+        '--keepdb', '--reverse', '--debug-sql',
+        # 1.9 arguments
+        '--parallel',
+        # 1.10 arguments
+        '--tag', '--exclude-tag',
+        # 1.11 arguments
+        '--debug-mode',
+    ]
 
-    django_opts = [opt.dest for opt in BaseCommand.option_list] + ['version']
-    return tuple(
-        o for o in options if o.dest not in django_opts and o.action != 'help')
+    #
+    # For optparse -> argparse conversion
+    #
+    # Option strings to remove from Django options if found
+    _argparse_remove_options = (
+        '-p',  # Short arg for nose's --plugins, not Django's --patterns
+        '-d',  # Short arg for nose's --detailed-errors, not Django's
+               #  --debug-sql
+    )
 
+    # Convert nose optparse options to argparse options
+    _argparse_type = {
+        'int': int,
+        'float': float,
+        'complex': complex,
+        'string': str,
+        'choice': str,
+    }
+    # If optparse has a None argument, omit from call to add_argument
+    _argparse_omit_if_none = (
+        'action', 'nargs', 'const', 'default', 'type', 'choices',
+        'required', 'help', 'metavar', 'dest')
 
-if hasattr(BaseCommand, 'use_argparse'):
-    # Django 1.8 and later uses argparse.ArgumentParser
-    # Translate nose optparse arguments to argparse
-    class BaseRunner(DiscoverRunner):
+    # Always ignore these optparse arguments
+    # Django will parse without calling the callback
+    # nose will then reparse with the callback
+    _argparse_callback_options = (
+        'callback', 'callback_args', 'callback_kwargs')
 
-        # Don't pass the following options to nosetests
-        django_opts = [
-            '--noinput', '--liveserver', '-p', '--pattern', '--testrunner',
-            '--settings',
-            # 1.8 arguments
-            '--keepdb', '--reverse', '--debug-sql',
-            # 1.9 arguments
-            '--parallel',
-        ]
+    # Keep track of nose options with nargs=1
+    _has_nargs = set(['--verbosity'])
 
-        #
-        # For optparse -> argparse conversion
-        #
-        # Option strings to remove from Django options if found
-        _argparse_remove_options = (
-            '-p',  # Short arg for nose's --plugins, not Django's --patterns
-            '-d',  # Short arg for nose's --detailed-errors, not Django's
-                   #  --debug-sql
-        )
+    @classmethod
+    def add_arguments(cls, parser):
+        """Convert nose's optparse arguments to argparse."""
+        super(BaseRunner, cls).add_arguments(parser)
 
-        # Convert nose optparse options to argparse options
-        _argparse_type = {
-            'int': int,
-            'float': float,
-            'complex': complex,
-            'string': str,
-            'choice': str,
-        }
-        # If optparse has a None argument, omit from call to add_argument
-        _argparse_omit_if_none = (
-            'action', 'nargs', 'const', 'default', 'type', 'choices',
-            'required', 'help', 'metavar', 'dest')
+        # Read optparse options for nose and plugins
+        cfg_files = nose.core.all_config_files()
+        manager = nose.core.DefaultPluginManager()
+        config = nose.core.Config(
+            env=os.environ, files=cfg_files, plugins=manager)
+        config.plugins.addPlugins(list(_get_plugins_from_settings()))
+        options = config.getParser()._get_all_options()
 
-        # Always ignore these optparse arguments
-        # Django will parse without calling the callback
-        # nose will then reparse with the callback
-        _argparse_callback_options = (
-            'callback', 'callback_args', 'callback_kwargs')
+        # Gather existing option strings`
+        django_options = set()
+        for action in parser._actions:
+            for override in cls._argparse_remove_options:
+                if override in action.option_strings:
+                    # Emulate parser.conflict_handler='resolve'
+                    parser._handle_conflict_resolve(
+                        None, ((override, action),))
+            django_options.update(action.option_strings)
 
-        # Keep track of nose options with nargs=1
-        _has_nargs = set(['--verbosity'])
+        # Process nose optparse options
+        for option in options:
+            # Gather options
+            opt_long = option.get_opt_string()
+            if option._short_opts:
+                opt_short = option._short_opts[0]
+            else:
+                opt_short = None
 
-        @classmethod
-        def add_arguments(cls, parser):
-            """Convert nose's optparse arguments to argparse."""
-            super(BaseRunner, cls).add_arguments(parser)
+            # Rename nose's --verbosity to --nose-verbosity
+            if opt_long == '--verbosity':
+                opt_long = '--nose-verbosity'
 
-            # Read optparse options for nose and plugins
-            cfg_files = nose.core.all_config_files()
-            manager = nose.core.DefaultPluginManager()
-            config = nose.core.Config(
-                env=os.environ, files=cfg_files, plugins=manager)
-            config.plugins.addPlugins(list(_get_plugins_from_settings()))
-            options = config.getParser()._get_all_options()
+            # Skip any options also in Django options
+            if opt_long in django_options:
+                continue
+            if opt_short and opt_short in django_options:
+                opt_short = None
 
-            # Gather existing option strings`
-            django_options = set()
-            for action in parser._actions:
-                for override in cls._argparse_remove_options:
-                    if override in action.option_strings:
-                        # Emulate parser.conflict_handler='resolve'
-                        parser._handle_conflict_resolve(
-                            None, ((override, action),))
-                django_options.update(action.option_strings)
-
-            # Process nose optparse options
-            for option in options:
-                # Skip any options also in Django options
-                opt_long = option.get_opt_string()
-                if opt_long in django_options:
+            # Convert optparse attributes to argparse attributes
+            option_attrs = {}
+            for attr in option.ATTRS:
+                # Ignore callback options
+                if attr in cls._argparse_callback_options:
                     continue
-                if option._short_opts:
-                    opt_short = option._short_opts[0]
-                    if opt_short in django_options:
-                        continue
-                else:
-                    opt_short = None
 
-                # Rename nose's --verbosity to --nose-verbosity
-                if opt_long == '--verbosity':
-                    opt_long = '--nose-verbosity'
+                value = getattr(option, attr)
 
-                # Convert optparse attributes to argparse attributes
-                option_attrs = {}
-                for attr in option.ATTRS:
-                    # Ignore callback options
-                    if attr in cls._argparse_callback_options:
-                        continue
+                if attr == 'default' and value == NO_DEFAULT:
+                    continue
 
-                    value = getattr(option, attr)
+                # Rename options for nose's --verbosity
+                if opt_long == '--nose-verbosity':
+                    if attr == 'dest':
+                        value = 'nose_verbosity'
+                    elif attr == 'metavar':
+                        value = 'NOSE_VERBOSITY'
 
-                    if attr == 'default' and value == NO_DEFAULT:
-                        continue
+                # Omit arguments that are None, use default
+                if attr in cls._argparse_omit_if_none and value is None:
+                    continue
 
-                    # Rename options for nose's --verbosity
-                    if opt_long == '--nose-verbosity':
-                        if attr == 'dest':
-                            value = 'nose_verbosity'
-                        elif attr == 'metavar':
-                            value = 'NOSE_VERBOSITY'
+                # Convert type from optparse string to argparse type
+                if attr == 'type':
+                    value = cls._argparse_type[value]
 
-                    # Omit arguments that are None, use default
-                    if attr in cls._argparse_omit_if_none and value is None:
-                        continue
+                # Convert action='callback' to action='store'
+                if attr == 'action' and value == 'callback':
+                    action = 'store'
 
-                    # Convert type from optparse string to argparse type
-                    if attr == 'type':
-                        value = cls._argparse_type[value]
+                # Keep track of nargs=1
+                if attr == 'nargs':
+                    assert value == 1, (
+                        'argparse option nargs=%s is not supported' %
+                        value)
+                    cls._has_nargs.add(opt_long)
+                    if opt_short:
+                        cls._has_nargs.add(opt_short)
 
-                    # Convert action='callback' to action='store'
-                    if attr == 'action' and value == 'callback':
-                        action = 'store'
+                # Pass converted attribute to optparse option
+                option_attrs[attr] = value
 
-                    # Keep track of nargs=1
-                    if attr == 'nargs':
-                        assert value == 1, (
-                            'argparse option nargs=%s is not supported' %
-                            value)
-                        cls._has_nargs.add(opt_long)
-                        if opt_short:
-                            cls._has_nargs.add(opt_short)
-
-                    # Pass converted attribute to optparse option
-                    option_attrs[attr] = value
-
-                # Add the optparse argument
-                if opt_short:
-                    parser.add_argument(opt_short, opt_long, **option_attrs)
-                else:
-                    parser.add_argument(opt_long, **option_attrs)
-else:
-    # Django 1.7 and earlier use optparse
-    class BaseRunner(DiscoverRunner):
-        # Replace the builtin options with the merged django/nose options:
-        options = _get_options()
-
-        # Not add following options to nosetests
-        django_opts = [
-            '--noinput', '--liveserver', '-p', '--pattern', '--testrunner']
-
-        # Default nosetest options with an argument
-        _has_nargs = set([
-            '--attr', '--config', '--cover-html-dir', '--cover-min-percentage',
-            '--cover-package', '--cover-xml-file', '--debug', '--debug-log',
-            '--doctest-extension', '--doctest-fixtures', '--doctest-options',
-            '--doctest-result-variable', '--eval-attr', '--exclude',
-            '--id-file', '--ignore-files', '--include', '--logging-config',
-            '--logging-datefmt', '--logging-filter', '--logging-format',
-            '--logging-level', '--match', '--process-timeout', '--processes',
-            '--profile-restrict', '--profile-sort', '--profile-stats-file',
-            '--py3where', '--tests', u'--verbosity', '--where', '--xunit-file',
-            '--xunit-testsuite-name', '-A', '-I', '-a', '-c', '-e', '-i', '-l',
-            '-m', '-w'])
+            # Add the optparse argument
+            if opt_short:
+                parser.add_argument(opt_short, opt_long, **option_attrs)
+            else:
+                parser.add_argument(opt_long, **option_attrs)
 
 
 class BasicNoseRunner(BaseRunner):
@@ -325,18 +239,15 @@ class BasicNoseRunner(BaseRunner):
         for plugin in _get_plugins_from_settings():
             plugins_to_add.append(plugin)
 
-        try:
-            django.setup()
-        except AttributeError:
-            # Setup isn't necessary in Django < 1.7
-            pass
+        setup()
 
         nose.core.TestProgram(argv=nose_argv, exit=False,
                               addplugins=plugins_to_add)
         return result_plugin.result
 
     def run_tests(self, test_labels, extra_tests=None):
-        """Run the unit tests for all the test names in the provided list.
+        """
+        Run the unit tests for all the test names in the provided list.
 
         Test names specified may be file or module names, and may optionally
         indicate the test case to run by separating the module or file name
@@ -349,7 +260,6 @@ class BasicNoseRunner(BaseRunner):
         not the whole string.
 
         Examples:
-
         runner.run_tests( ('test.module',) )
         runner.run_tests(['another.test:TestCase.test_method'])
         runner.run_tests(['a.test:TestCase'])
@@ -361,17 +271,12 @@ class BasicNoseRunner(BaseRunner):
         but the extra tests will not be run.  Maybe later.
 
         Returns the number of tests that failed.
+
         """
         nose_argv = (['nosetests'] + list(test_labels))
         if hasattr(settings, 'NOSE_ARGS'):
             nose_argv.extend(settings.NOSE_ARGS)
 
-        # Skip over 'manage.py test' and any arguments handled by django.
-        django_opts = self.django_opts[:]
-        for opt in BaseCommand.option_list:
-            django_opts.extend(opt._long_opts)
-            django_opts.extend(opt._short_opts)
-
         # Recreate the arguments in a nose-compatible format
         arglist = sys.argv[1:]
         has_nargs = getattr(self, '_has_nargs', set(['--verbosity']))
@@ -380,7 +285,7 @@ class BasicNoseRunner(BaseRunner):
             if not opt.startswith('-'):
                 # Discard test labels
                 continue
-            if any(opt.startswith(d) for d in django_opts):
+            if any(opt.startswith(d) for d in self.django_opts):
                 # Discard options handled by Djangp
                 continue
 
@@ -531,7 +436,7 @@ class NoseTestSuiteRunner(BasicNoseRunner):
                 m._meta.db_table in tables]
 
     def setup_databases(self):
-        """Setup databases, skipping DB creation if requested and possible."""
+        """Set up databases. Skip DB creation if requested and possible."""
         for alias in connections:
             connection = connections[alias]
             creation = connection.creation
