@@ -1,9 +1,12 @@
+"""Additional plugins for testing plugins."""
 from nose.plugins import Plugin
 
-
 plugin_began = False
 
+
 class SanityCheckPlugin(Plugin):
+    """Test plugin that registers that it ran."""
+
     enabled = True
 
     def options(self, parser, env):
@@ -13,5 +16,6 @@ class SanityCheckPlugin(Plugin):
         """Configure plugin."""
 
     def begin(self):
+        """Flag that the plugin was run."""
         global plugin_began
         plugin_began = True
